@@ -40,6 +40,12 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 
+#include <extensions/cgi/cgroupuse.h>
+#include <extensions/cgi/cgroupconn.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <pwd.h>
+
 extern char *argv0;
 
 CgidWorker *CgidWorker::s_pCgid = NULL;
@@ -50,7 +56,6 @@ CgidWorker::CgidWorker(const char *pName)
     : ExtWorker(HandlerType::HT_CGI)
     , m_pid(-1)
     , m_fdCgid(-1)
-    , m_lve(1)
 {
     setConfigPointer(new CgidConfig(pName));
 }
@@ -153,7 +158,10 @@ int CgidWorker::spawnCgid(int fd, char *pData, const char *secret)
     //in child
     if (pid == 0)
     {
+        char lve_env[16];
         CloseUnusedFd(fd);
+        snprintf(lve_env, sizeof(lve_env) -1, "LVE_ENABLE=%d", getLVE());
+        putenv(lve_env);
         int ret = lscgid_main(fd, argv0, secret, pData);
         exit(ret);
     }
@@ -210,6 +218,60 @@ int CgidWorker::watchDog(const char *pServerRoot, const char *pChroot,
 }
 
 
+#if defined(linux) || defined(__linux) || defined(__linux__) || defined(__gnu_linux__)
+static int cgroup_validate(void)
+{
+    struct passwd *pwd;
+    
+    if (getuid())
+    {
+        LS_INFO("You allow cgroups, but are not running as root; disabled.\n");
+        return -1;
+    }
+    pwd = getpwnam("nobody");
+    if (pwd)
+    {
+        int pid = fork();
+        if (pid == 0)
+        {
+            // child
+            CGroupConn conn;
+            if (!(conn.create()))
+            {
+                CGroupUse use(&conn);
+                int apply = 0;
+                int validate = 0;
+                if ((!(apply = use.apply(pwd->pw_uid))) &&
+                    (!(validate = use.validate())))
+                {
+                    exit(0);
+                }
+            }
+            exit(1);
+        }
+        if (pid > 0)
+        { 
+            int result;
+            waitpid(pid, &result, 0);
+            if (WIFEXITED(result) && (WEXITSTATUS(result) == 0))
+            {
+                LS_DBG_H("Ok to run cgroups\n");
+                return 0;
+            }
+        }
+    }
+    else {
+        LS_INFO("To validate cgroups, 'nobody' user required and not found\n");
+        return -1;
+    }
+    LS_INFO("You allow cgroups, but your OS does not support it; disabled.  "
+            "Try installing glib\n");
+    return -1;
+}
+#endif
+
+
+
 int CgidWorker::config(const XmlNode *pNode1)
 {
     int iChrootlen = 0;
@@ -241,7 +303,7 @@ int CgidWorker::config(const XmlNode *pNode1)
 
     if (!pValue)
     {
-        snprintf(achSocket, 128, "uds:/%s%sadmin/cgid/cgid.sock",
+        snprintf(achSocket, 128, "uds:/%s%scgid/cgid.sock",
                  (iChrootlen) ? psChroot : "",
                  MainServerConfig::getInstance().getServerRoot());
     }
@@ -290,7 +352,17 @@ int CgidWorker::config(const XmlNode *pNode1)
    HttpMime::getMime()->addMimeHandler("", achMIME_SSI,
                     HandlerFactory::getInstance(HandlerType::HT_SSI, NULL),
                     NULL, TmpLogId::getLogId());
-
+   
+#if defined(linux) || defined(__linux) || defined(__linux__) || defined(__gnu_linux__)
+    procConfig.setCGroupAllow(ConfigCtx::getCurConfigCtx()->getLongValue(pNode1,
+        "cgroups", 0, 2, 0) != ServerProcessConfig::CGROUP_CONFIG_DISALLOW);
+    if ((procConfig.getCGroupAllow()) && (cgroup_validate() == -1))
+        procConfig.setCGroupAllow(0);
+    
+    procConfig.setCGroupDefault(ConfigCtx::getCurConfigCtx()->getLongValue(pNode1,
+        "cgroups", 0, 2, 0) == ServerProcessConfig::CGROUP_CONFIG_DEFAULT_ON);
+#endif
+    
     CgidWorker::setCgidWorkerPid(
         start(MainServerConfig::getInstance().getServerRoot(), psChroot,
               procConfig.getUid(), procConfig.getGid(),
@@ -298,6 +370,10 @@ int CgidWorker::config(const XmlNode *pNode1)
     return CgidWorker::getCgidWorkerPid();
 }
 
+int  CgidWorker::getLVE() const
+{
+    return HttpServerConfig::getInstance().getEnableLve();
+}
 
 void CgidWorker::closeFdCgid()
 {
