@@ -20,74 +20,19 @@
 #include "h2connection.h"
 #include "unpackedheaders.h"
 
+#include <http/hiohandlerfactory.h>
+
 #include <util/datetime.h>
 #include <log4cxx/logger.h>
 #include <lsr/ls_strtool.h>
 #include <util/iovec.h>
+#include <util/ssnprintf.h>
 #include "lsdef.h"
 
 H2Stream::H2Stream()
-    : m_pH2Conn(NULL)
-    , m_bufIn(0)
-    , m_uiStreamId(0)
-    , m_iWindowOut(H2_FCW_INIT_SIZE)
-    , m_iWindowIn(H2_FCW_INIT_SIZE)
+    : m_bufIn(0)
 {
-}
-
-
-const char *H2Stream::buildLogId()
-{
-    m_logId.len = ls_snprintf(m_logId.ptr, MAX_LOGID_LEN, "%s-%d",
-                      m_pH2Conn->getStream()->getLogId(), m_uiStreamId);
-    return m_logId.ptr;
-}
-
-
-int H2Stream::init(uint32_t StreamID, H2Connection *pH2Conn,
-                   HioHandler *pHandler, Priority_st *pPriority)
-{
-    HioStream::reset(DateTime::s_curTime);
-    pHandler->attachStream(this);
-    clearLogId();
-
-    setState(HIOS_CONNECTED);
-
-    m_bufIn.clear();
-    m_uiStreamId  = StreamID;
-    m_iWindowOut = pH2Conn->getStreamOutInitWindowSize();
-    m_iWindowIn = pH2Conn->getStreamInInitWindowSize();
-
-    int pri = HIO_PRIORITY_HTML;
-    if (pPriority)
-    {
-        if (pPriority->m_weight <= 32)
-            pri = (32 - pPriority->m_weight) >> 2;
-        else
-            pri = (256 - pPriority->m_weight) >> 5;
-    }
-    setPriority(pri);
-
-    m_pH2Conn = pH2Conn;
-    LS_DBG_L(this, "H2Stream::init(), id: %d, priority: %d, flag: %d. ",
-             StreamID, pri, (int)getFlag());
-    return 0;
-}
-
-
-int H2Stream::onInitConnected(bool bUpgraded)
-{
-    if (!bUpgraded)
-        getHandler()->onInitConnected();
-    
-    if (isWantRead())
-        getHandler()->onReadEx();
-    if (isWantWrite())
-        if (next() == NULL)
-            m_pH2Conn->add2PriorityQue(this);
-
-    //getHandler()->onWriteEx();
-    return 0;
+    LS_ZERO_FILL(m_pH2Conn, m_iWindowIn);
 }
 
 
@@ -96,12 +41,102 @@ H2Stream::~H2Stream()
     m_bufIn.clear();
 }
 
-//#define FOR_DEBUG_H2_BUFFER_DATA
+
+void H2Stream::reset()
+{
+    HioStream::reset(0);
+    m_bufIn.clear();
+    m_headers.reset();
+    LS_ZERO_FILL(m_pH2Conn, m_iWindowIn);
+}
+
+const char *H2Stream::buildLogId()
+{
+    m_logId.len = safe_snprintf(m_logId.ptr, MAX_LOGID_LEN, "%s-%d",
+                        m_pH2Conn->getStream()->getLogId(), getStreamID());
+    return m_logId.ptr;
+}
+
+
+void H2Stream::apply_priority(Priority_st *pPriority)
+{
+    int pri = HIO_PRIORITY_HTML;
+    if (pPriority)
+    {
+        if (pPriority->m_weight <= 32)
+            pri = (32 - pPriority->m_weight) >> 2;
+        else
+            pri = (256 - pPriority->m_weight) >> 5;
+    }
+    if (pri != getPriority())
+    {
+        if (next())
+        {
+            m_pH2Conn->removePriQue(this);
+            setPriority(pri);
+            m_pH2Conn->add2PriorityQue(this);
+        }
+        else
+            setPriority(pri);
+    }
+}
+
+
+int H2Stream::init(H2Connection *pH2Conn,
+                   Priority_st *pPriority)
+{
+    setActiveTime(DateTime::s_curTime);
+    clearLogId();
+
+    setState(HIOS_CONNECTED);
+    uint32_t flag = HIO_FLAG_WRITE_BUFFER | HIO_FLAG_SENDFILE;
+    if (pH2Conn->getStream()->getFlag(HIO_FLAG_ALTSVC_SENT))
+        flag |= HIO_FLAG_ALTSVC_SENT;
+    setFlag(flag, 1);
+    m_bufIn.clear();
+    m_pH2Conn = pH2Conn;
+    m_iWindowOut = pH2Conn->getStreamOutInitWindowSize();
+    m_iWindowIn = pH2Conn->getStreamInInitWindowSize();
+
+    apply_priority(pPriority);
+
+    LS_DBG_L(this, "H2Stream::init(), id: %d, priority: %d, flag: %d. ",
+             getStreamID(), getPriority(), (int)getFlag());
+    return 0;
+}
+
+
+int H2Stream::onInitConnected(HioHandler *pHandler, bool bUpgraded)
+{
+    assert(getHandler() == NULL);
+    if (!pHandler)
+    {
+        pHandler = HioHandlerFactory::getHandler(HIOS_PROTO_HTTP);
+        if (!pHandler)
+        {
+            m_pH2Conn->resetStream(0, this, H2_ERROR_INTERNAL_ERROR);
+            return LS_FAIL;
+        }
+    }
+    if (pHandler)
+    {
+        pHandler->attachStream(this);
+    }
+    if (!bUpgraded)
+        getHandler()->onInitConnected();
+
+    if (isWantRead())
+        getHandler()->onReadEx();
+    if (isWantWrite())
+        if (next() == NULL)
+            m_pH2Conn->add2PriorityQue(this);
+    //getHandler()->onWriteEx();
+    return 0;
+}
+
+
 int H2Stream::appendReqData(char *pData, int len, uint8_t flags)
 {
-#ifdef FOR_DEBUG_H2_BUFFER_DATA
-    static bool readstart = 0;
-#endif
     if (m_bufIn.append(pData, len) == -1)
         return LS_FAIL;
     if (isFlowCtrl())
@@ -110,15 +145,17 @@ int H2Stream::appendReqData(char *pData, int len, uint8_t flags)
     //      H2_CTRL_FLAG_UNIDIRECTIONAL is directly mapped to HIO_FLAG_LOCAL_SHUTDOWN
     if (flags & H2_CTRL_FLAG_FIN)
     {
-        setFlag(H2_CTRL_FLAG_FIN, 1);
+        setFlag(HIO_FLAG_PEER_SHUTDOWN, 1);
+        if (getFlag(HIO_FLAG_INIT_SESS))
+        {
+            setFlag(HIO_FLAG_INIT_SESS, 0);
+            if (onInitConnected(NULL, 0) == LS_FAIL)
+                return len;
+        }
         if (getHandler()->detectContentLenMismatch(m_bufIn.size()))
             return LS_FAIL;
     }
-    if (isWantRead()
-#ifdef FOR_DEBUG_H2_BUFFER_DATA
-        && (readstart || (readstart = (m_bufIn.size() > 10000000)))
-#endif
-    )
+    if (isWantRead() && getHandler())
         getHandler()->onReadEx();
     return len;
 }
@@ -173,41 +210,56 @@ void H2Stream:: continueWrite()
 
 int H2Stream::onTimer()
 {
-    if (getState() == HIOS_CONNECTED)
+    if (getState() == HIOS_CONNECTED && getHandler())
         return getHandler()->onTimerEx();
     return 0;
 }
 
 
-uint16_t H2Stream::getEvents() const
+void H2Stream::shutdownEx()
 {
-    return m_pH2Conn->getEvents();
+    LS_DBG_L(this, "H2Stream::shutdown()");
+    m_pH2Conn->sendFinFrame(getStreamID());
+    setActiveTime(DateTime::s_curTime);
+    m_pH2Conn->wantFlush();
 }
 
-
-int H2Stream::isFromLocalAddr() const
-{   return m_pH2Conn->isFromLocalAddr();  }
-
-
-NtwkIOLink *H2Stream::getNtwkIoLink()
-{
-    return m_pH2Conn->getNtwkIoLink();
-}
-
-
 int H2Stream::shutdown()
 {
     if (getState() >= HIOS_SHUTDOWN)
         return 0;
 
+    shutdownEx();
+
+    markShutdown();
+    return 0;
+}
+
+
+void H2Stream::markShutdown()
+{
     setState(HIOS_SHUTDOWN);
     m_pH2Conn->incShutdownStream();
+    if (!next())
+    {
+        setPriority(0);
+        m_pH2Conn->add2PriorityQue(this);
+    }
+}
 
-    LS_DBG_L(this, "H2Stream::shutdown()");
-    m_pH2Conn->sendFinFrame(m_uiStreamId);
-    setActiveTime(DateTime::s_curTime);
-    m_pH2Conn->wantFlush();
-    return 0;
+
+void H2Stream::closeEx()
+{
+    if (getState() == HIOS_DISCONNECTED)
+        return;
+    if (getHandler() && !isReadyToRelease())
+        getHandler()->onCloseEx();
+    if (getState() < HIOS_SHUTDOWN)
+        shutdownEx();
+    if (getState() == HIOS_SHUTDOWN)
+        m_pH2Conn->decShutdownStream();
+    setState(HIOS_DISCONNECTED);
+    setFlag(HIO_FLAG_WANT_WRITE, 0);
 }
 
 
@@ -215,20 +267,8 @@ int H2Stream::close()
 {
     if (getState() == HIOS_DISCONNECTED)
         return 0;
-    if (getHandler() && !isReadyToRelease())
-        getHandler()->onCloseEx();
-    if (getState() < HIOS_SHUTDOWN)
-        shutdown();
-    if (getState() == HIOS_SHUTDOWN)
-        m_pH2Conn->decShutdownStream();
-    setState(HIOS_DISCONNECTED);
-    setFlag(HIO_FLAG_WANT_WRITE, 0);
-    //if (getHandler())
-    //{
-    //    getHandler()->recycle();
-    //    setHandler( NULL );
-    //}
-    m_pH2Conn->recycleStream(m_uiStreamId);
+    closeEx();
+    m_pH2Conn->recycleStream(this);
     return 0;
 }
 
@@ -242,10 +282,9 @@ int H2Stream::flush()
 
 int H2Stream::getDataFrameSize(int wanted)
 {
-    if ((m_pH2Conn->isOutBufFull()) ||
-        (0 >= m_iWindowOut))
+    if ((m_pH2Conn->isOutBufFull()) || (0 >= m_iWindowOut))
     {
-        setFlag(HIO_FLAG_BUFF_FULL | HIO_FLAG_WANT_WRITE, 1);
+        setFlag(HIO_FLAG_PAUSE_WRITE | HIO_FLAG_WANT_WRITE, 1);
         if (next() == NULL)
             m_pH2Conn->add2PriorityQue(this);
         m_pH2Conn->setPendingWrite();
@@ -261,33 +300,61 @@ int H2Stream::getDataFrameSize(int wanted)
 
 int H2Stream::writev(IOVec &vector, int total)
 {
-    int size;
     int ret;
+    int pkt_size;
     if (getState() == HIOS_DISCONNECTED)
         return LS_FAIL;
-    if (getFlag(HIO_FLAG_BUFF_FULL))
-        return 0;
-    size = getDataFrameSize(total);
-    if (size <= 0)
-        return 0;
-    if (size < total)
-    {
-        //adjust vector
-        IOVec iov(vector);
-        total = iov.shrinkTo(size, 0);
-        ret = sendData(&iov, size);
-    }
-    else
-        ret = sendData(&vector, size);
-    if (ret == -1)
-        return LS_FAIL;
-    return size;
+    int weight = m_pH2Conn->getWeightedPriority(this);
+    LS_DBG_L(this, "H2Stream::writev(%d, %d), weight: %d", vector.len(),
+             total, weight);
+    int flag = 0;
+    int remain = total;
+    int fin = 0;
+    IOVec iov(vector);
 
+    while(!isPauseWrite() && remain > 0)
+    {
+        pkt_size = getDataFrameSize(remain);
+        if (pkt_size <= 0)
+            break;
+        if (total - remain + pkt_size >= (128 * 1024) / weight)
+        {
+            if (pkt_size > 16384)
+            {
+                int new_pkt_size = (128 * 1024) / weight - (total - remain);
+                if (new_pkt_size < 16384)
+                    pkt_size = 16384;
+                else
+                    pkt_size = new_pkt_size;
+            }
+            LS_DBG_L(this, "H2Stream::write() limited by weight, pkt_size: %d", pkt_size);
+            if (pkt_size <= 0)
+                break;
+            setFlag(HIO_FLAG_PAUSE_WRITE, 1);
+        }
+        if (pkt_size >= remain && (flag & HIO_EOR))
+        {
+            fin = H2_FLAG_END_STREAM;
+            markShutdown();
+        }
+        ret = m_pH2Conn->sendDataFrame(getStreamID(), fin, &iov, pkt_size);
+        LS_DBG_L(this, "H2Stream::sendData(), total: %d, ret: %d", total, ret);
+        if (fin)
+            m_pH2Conn->wantFlush();
+        ret = dataSent(ret);
+        if (ret <= 0)
+            break;
+        remain -= ret;
+    }
+    return total - remain;
 }
 
 
 int H2Stream::writev(const struct iovec *vec, int count)
 {
+    if (count == 1)
+        return write((const char *)vec->iov_base, vec->iov_len, 0);
+
     IOVec iov(vec, count);
     return writev(iov, iov.bytes());
 }
@@ -295,26 +362,146 @@ int H2Stream::writev(const struct iovec *vec, int count)
 
 int H2Stream::write(const char *buf, int len)
 {
-    int allowed;
+    return write(buf, len, 0);
+}
+
+
+// int H2Stream::write(const char *buf, int len, int flag)
+// {
+//     int allowed;
+//     if (getState() == HIOS_DISCONNECTED)
+//         return LS_FAIL;
+//     allowed = getDataFrameSize(len);
+//     if (allowed <= 0)
+//         return 0;
+//     if (allowed >= len && (flag & HIO_EOR))
+//     {
+//         flag = H2_FLAG_END_STREAM;
+//         markShutdown();
+//     }
+//     else
+//         flag = 0;
+//     int ret = m_pH2Conn->sendDataFrame(m_uiStreamId, flag, buf, allowed);
+//     LS_DBG_L(this, "H2Stream::write(%p, %d, %d) return %d", buf, allowed, flag, ret);
+//     if (flag)
+//         m_pH2Conn->wantFlush();
+//     return dataSent(ret);
+// }
+
+
+int H2Stream::write(const char *buf, int len, int flag)
+{
+    const char *end = buf + len;
+    const char *p = buf;
+    int fin = 0;
+    int ret = 0;
+    int pkt_size;
+    int weight;
     if (getState() == HIOS_DISCONNECTED)
         return LS_FAIL;
-    allowed = getDataFrameSize(len);
-    if (allowed <= 0)
-        return 0;
+    weight = m_pH2Conn->getWeightedPriority(this);
+    LS_DBG_L(this, "H2Stream::write(%p, %d, %d), weight: %d", buf, len,
+             flag, weight);
 
-    int ret = m_pH2Conn->sendDataFrame(m_uiStreamId, 0, buf, allowed);
-    return dataSent(ret);
+    while(!isPauseWrite() && end - p > 0)
+    {
+        pkt_size = getDataFrameSize(end - p);
+        if (pkt_size <= 0)
+            break;
+        if (p + pkt_size - buf >= (128 * 1024) / weight)
+        {
+            if (pkt_size > 16384)
+            {
+                int new_pkt_size = (128 * 1024) / weight - (p - buf);
+                if (new_pkt_size < 16384)
+                    pkt_size = 16384;
+                else
+                    pkt_size = new_pkt_size;
+            }
+            LS_DBG_L(this, "H2Stream::write() limited by weight, pkt_size: %d", pkt_size);
+            if (pkt_size <= 0)
+                break;
+            setFlag(HIO_FLAG_PAUSE_WRITE, 1);
+        }
+        if (pkt_size >= end - p && (flag & HIO_EOR))
+        {
+            fin = H2_FLAG_END_STREAM;
+            markShutdown();
+        }
+        ret = m_pH2Conn->sendDataFrame(getStreamID(), fin, p, pkt_size);
+        LS_DBG_L(this, "H2Stream::write(%p, %d, %d) return %d", p, pkt_size, fin, ret);
+        if (fin)
+            m_pH2Conn->wantFlush();
+        ret = dataSent(ret);
+        if (ret <= 0)
+            break;
+        p += ret;
+    }
+    return (p - buf > 0)? p - buf : ret;
+}
+
+
+int H2Stream::sendfile(int fdSrc, off_t off, size_t size, int flag)
+{
+    off_t cur = off;
+    off_t end = off + size;
+    int fin = 0;
+    int ret = 0;
+    int pkt_size;
+    int weight;
+    if (getState() == HIOS_DISCONNECTED)
+        return LS_FAIL;
+    weight = m_pH2Conn->getWeightedPriority(this);
+    LS_DBG_L(this, "H2Stream::sendfile(%d, %lld, %zd, %d), weight: %d",
+             fdSrc, (long long)off, size, flag, weight);
+
+    while(!isPauseWrite() && end - cur > 0)
+    {
+        pkt_size = getDataFrameSize(end - cur);
+        if (pkt_size <= 0)
+            break;
+        if (cur + pkt_size - off >= (128 * 1024) / weight)
+        {
+            if (pkt_size > 16384)
+            {
+                int new_pkt_size = (128 * 1024) / weight - (cur - off);
+                if (new_pkt_size < 16384)
+                    pkt_size = 16384;
+                else
+                    pkt_size = new_pkt_size;
+            }
+            LS_DBG_L(this, "H2Stream::sendfile() limited by weight, pkt_size: %d", pkt_size);
+            if (pkt_size <= 0)
+                break;
+            setFlag(HIO_FLAG_PAUSE_WRITE, 1);
+        }
+//         if (pkt_size == 16384)
+//             pkt_size -= 9;
+        if (pkt_size >= end - cur && (flag & HIO_EOR))
+        {
+            fin = H2_FLAG_END_STREAM;
+            markShutdown();
+        }
+        ret = m_pH2Conn->sendfileDataFrame(getStreamID(), fin, fdSrc, cur, pkt_size);
+        LS_DBG_L(this, "H2Stream::sendfileDataFrame(%d, %lld, %d, %d) return %d",
+                 fdSrc, (long long)cur, pkt_size, fin, ret);
+        if (fin)
+            m_pH2Conn->wantFlush();
+        ret = dataSent(ret);
+        if (ret <= 0)
+            break;
+        cur += ret;
+    }
+    return (cur - off > 0)? cur - off : ret;
 }
 
 
 int H2Stream::onWrite()
 {
     LS_DBG_L(this, "H2Stream::onWrite()");
-    if (m_pH2Conn->isOutBufFull())
-        return 0;
     if (m_iWindowOut <= 0)
         return 0;
-    setFlag(HIO_FLAG_BUFF_FULL, 0);
+    setFlag(HIO_FLAG_PAUSE_WRITE, 0);
 
     if (isWantWrite())
     {
@@ -331,15 +518,6 @@ int H2Stream::onWrite()
 }
 
 
-int H2Stream::sendData(IOVec *pIov, int total)
-{
-    int ret;
-    ret = m_pH2Conn->sendDataFrame(m_uiStreamId, 0, pIov, total);
-    LS_DBG_L(this, "H2Stream::sendData(), total: %d, ret: %d", total, ret);
-    return dataSent(ret);
-}
-
-
 int H2Stream::dataSent(int ret)
 {
     if (ret == -1)
@@ -357,7 +535,7 @@ int H2Stream::dataSent(int ret)
                  (long long)getBytesSent(), m_iWindowOut);
 
         if (m_iWindowOut <= 0)
-            setFlag(HIO_FLAG_BUFF_FULL, 1);
+            setFlag(HIO_FLAG_PAUSE_WRITE, 1);
     }
     return ret;
 }
@@ -374,11 +552,20 @@ int H2Stream::sendRespHeaders(HttpRespHeaders *pHeaders, int isNoBody)
         flag |= H2_FLAG_END_STREAM;
         if (getState() != HIOS_SHUTDOWN)
         {
-            setState(HIOS_SHUTDOWN);
-            m_pH2Conn->incShutdownStream();
+            markShutdown();
         }
     }
-    return m_pH2Conn->sendRespHeaders(pHeaders, m_uiStreamId, flag);
+//     else
+//     {
+//         if (next() == NULL)
+//             m_pH2Conn->add2PriorityQue(this);
+//     }
+    int ret = m_pH2Conn->sendRespHeaders(pHeaders, getStreamID(), flag);
+    if (isNoBody)
+        m_pH2Conn->wantFlush();
+    if (ret != LS_FAIL && getFlag(HIO_FLAG_ALTSVC_SENT))
+        m_pH2Conn->getStream()->setFlag(HIO_FLAG_ALTSVC_SENT, 1);
+    return ret;
 }
 
 
@@ -404,7 +591,9 @@ int H2Stream::adjWindowOut(int32_t n)
 int H2Stream::push(ls_str_t *pUrl, ls_str_t *pHost, 
                    ls_strpair_t *pExtraHeaders)
 {
-    return m_pH2Conn->pushPromise(m_uiStreamId, pUrl, pHost, 
+    return m_pH2Conn->pushPromise(getStreamID(), pUrl, pHost,
                                   pExtraHeaders);
 }
 
+
+
